import React, { useState, useEffect } from 'react';
import ReactDOM from 'react-dom';
import { csv, scaleBand, scaleLinear, max } from 'd3';

const csvUrl =
  'https://gist.githubusercontent.com/YaminiSreeram/64b9cef153989a4632aea1b5787d2642/raw/3352cb06d21eece602e44315a1d7742b26d39a0b/cereals.csv';
const width = 800;
const height = 400;
const margin = { top: 20, right: 30, bottom: 50, left: 60 };

const App = () => {
  const [data, setData] = useState(null);

  useEffect(() => {
    const row = (d) => {
      d.Calories = +d['Calories'];
      return d;
    };
    csv(csvUrl, row).then((data) => {
      setData(data);
    });
  }, []);

  if (!data) {
    return <pre>Loading...</pre>;
  }

  const innerHeight = height - margin.top - margin.bottom;
  const innerWidth = width - margin.left - margin.right;

  const xScale = scaleBand()
    .domain(data.map((d) => d['Cereal Name']))
    .range([0, innerWidth])
    .padding(0.1);

  const yScale = scaleLinear()
    .domain([0, max(data, (d) => d.Calories)])
    .nice()
    .range([innerHeight, 0]);

  return (
    <svg width={width} height={height}>
      <g
        transform={`translate(${margin.left},${margin.top})`}
      >
        {data.map((d) => (
          <rect
            key={d['Cereal Name']}
            x={xScale(d['Cereal Name'])}
            y={yScale(d.Calories)}
            width={xScale.bandwidth()}
            height={innerHeight - yScale(d.Calories)}
            fill="Green"
          />
        ))}
        <g className="axis" transform={`translate(0,0)`}>
          {yScale.ticks().map((tickValue) => (
            <g
              key={tickValue}
              transform={`translate(0,${yScale(tickValue)})`}
            >
              <line x2={innerWidth} stroke="#ccc" />
              <text
                x="-6"
                dy="0.32em"
                textAnchor="end"
                fill="#000"
                fontSize="12"
              >
                {tickValue}
              </text>
            </g>
          ))}
        </g>
        <g
          className="axis"
          transform={`translate(0,${innerHeight})`}
        >
          {xScale.domain().map((cereal) => (
            <g
              key={cereal}
              transform={`translate(${xScale(cereal) + xScale.bandwidth() / 2},${margin.bottom - 10})`}
            >
              <line y2="6" stroke="#000" />
              <text
                y="9"
                dy="0.71em"
                textAnchor="middle"
                fill="#000"
                transform="rotate(-45)"
                fontSize="10"
              >
                {cereal}
              </text>
            </g>
          ))}
        </g>
      </g>
    </svg>
  );
};

const rootElement = document.getElementById('root');
ReactDOM.render(<App />, rootElement);
